var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __objRest = (source, exclude) => {
  var target = {};
  for (var prop in source)
    if (__hasOwnProp.call(source, prop) && exclude.indexOf(prop) < 0)
      target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(source)) {
      if (exclude.indexOf(prop) < 0 && __propIsEnum.call(source, prop))
        target[prop] = source[prop];
    }
  return target;
};

// src/leafs.ts
var StaticText = class {
  constructor(text) {
    this.text = text;
  }
  toString() {
    if (this.text === null) {
      return "null";
    }
    return this.text;
  }
  // isEqual(other: StaticText): boolean {
  //   if (this.text === null || other.text === null) {
  //     return this.text === other.text;
  //   } else {
  //     if (this.text instanceof RegExp && other.text instanceof RegExp) {
  //       return this.text.toString() === other.text.toString();
  //     } else if (this.text instanceof RegExp) {
  //       return this.text.test(other.text);
  //     } else if (other.text instanceof RegExp) {
  //       return other.text.test(this.text);
  //     } else {
  //       return this.text === other.text;
  //     }
  //   }
  // }
};

// src/role-helpers.ts
import { elementRoles } from "aria-query";

// src/type-guards.ts
var isOptionElement = (element) => element.tagName === "OPTION";
var isInputElement = (element) => element.tagName === "INPUT";
var isDetailsElement = (element) => element.tagName === "DETAILS";
var isDefined = (value) => value !== void 0 && value !== null;
var isA11yTreeNode = (value) => typeof value === "object" && value !== null && "name" in value && "role" in value;
var isA11yTreeNodeMatch = (value) => typeof value === "object" && value !== null && ("name" in value || "role" in value || "description" in value || "state" in value || "children" in value);

// src/virtual-roles.ts
var nonLandmarkVirtualRoles = [
  // header: 'HeaderAsNonLandmark',
  {
    match: (node) => {
      return node.tagName.toLowerCase() === "header";
    },
    roles: ["HeaderAsNonLandmark"],
    specificity: 1
  },
  // footer: 'FooterAsNonLandmark',
  {
    match: (node) => node.tagName.toLowerCase() === "footer",
    roles: ["FooterAsNonLandmark"],
    specificity: 1
  }
];
var virtualRoles = [
  // details: 'Details',
  {
    match: (node) => node.tagName.toLowerCase() === "details",
    roles: ["Details"],
    specificity: 1
  },
  // dd: 'DescriptionListDetails',
  {
    match: (node) => node.tagName.toLowerCase() === "dd",
    roles: ["DescriptionListDetails"],
    specificity: 1
  },
  // dl: 'DescriptionList',
  {
    match: (node) => node.tagName.toLowerCase() === "dl",
    roles: ["DescriptionList"],
    specificity: 1
  },
  // dt: 'DescriptionListTerm',
  {
    match: (node) => node.tagName.toLowerCase() === "dt",
    roles: ["DescriptionListTerm"],
    specificity: 1
  },
  // embed: 'EmbeddedObject',
  {
    match: (node) => node.tagName.toLowerCase() === "embed",
    roles: ["EmbeddedObject"],
    specificity: 1
  },
  // object: 'PluginObject',
  {
    match: (node) => node.tagName.toLowerCase() === "object",
    roles: ["PluginObject"],
    specificity: 1
  },
  // label: 'LabelText',
  {
    match: (node) => node.tagName.toLowerCase() === "label",
    roles: ["LabelText"],
    specificity: 1
  },
  // summary: 'DisclosureTriangle',
  {
    match: (node) => node.tagName.toLowerCase() === "summary",
    roles: ["DisclosureTriangle"],
    specificity: 1
  }
];

// src/role-helpers.ts
var elementRoleList = buildElementRoleList(elementRoles);
function getImplicitAriaRoles(currentNode, isNonLandmarkSubtree) {
  let result = [];
  for (const { match, roles } of elementRoleList) {
    if (match(currentNode)) {
      result = [...roles];
      break;
    }
  }
  if (isNonLandmarkSubtree) {
    for (const { match, roles } of nonLandmarkVirtualRoles) {
      if (match(currentNode)) {
        result = [...roles];
        break;
      }
    }
  }
  if (result.length === 0) {
    for (const { match, roles } of virtualRoles) {
      if (match(currentNode)) {
        result = [...roles];
        break;
      }
    }
  }
  return result;
}
function buildElementRoleList(elementRolesMap) {
  function makeElementSelector({ name, attributes }) {
    return `${name}${(attributes != null ? attributes : []).map(({ name: attributeName, value, constraints = [] }) => {
      const shouldNotExist = constraints.indexOf("undefined") !== -1;
      if (shouldNotExist) {
        return `:not([${attributeName}])`;
      } else if (value) {
        return `[${attributeName}="${value}"]`;
      } else {
        return `[${attributeName}]`;
      }
    }).join("")}`;
  }
  function getSelectorSpecificity({
    attributes = []
  }) {
    return attributes.length;
  }
  function bySelectorSpecificity({ specificity: leftSpecificity }, { specificity: rightSpecificity }) {
    return rightSpecificity - leftSpecificity;
  }
  function match(element) {
    let { attributes = [] } = element;
    const typeTextIndex = attributes.findIndex(
      (attribute) => attribute.value && attribute.name === "type" && attribute.value === "text"
    );
    if (typeTextIndex >= 0) {
      attributes = [
        ...attributes.slice(0, typeTextIndex),
        ...attributes.slice(typeTextIndex + 1)
      ];
    }
    const selector = makeElementSelector(__spreadProps(__spreadValues({}, element), { attributes }));
    return (node) => {
      if (typeTextIndex >= 0 && isInputElement(node) && node.type !== "text") {
        return false;
      }
      return node.matches(selector);
    };
  }
  let result = [];
  for (const [element, roles] of elementRolesMap.entries()) {
    result = [
      ...result,
      {
        match: match(element),
        roles: Array.from(roles),
        specificity: getSelectorSpecificity(element)
      }
    ];
  }
  return result.sort(bySelectorSpecificity);
}

// src/compute-properties.ts
function checkBooleanAttribute(element, attribute) {
  const attributeValue = element.getAttribute(attribute);
  if (attributeValue === "true") {
    return true;
  }
  if (attributeValue === "false") {
    return false;
  }
  return void 0;
}
function computeAriaSelected(element) {
  if (isOptionElement(element)) {
    return element.selected;
  }
  return checkBooleanAttribute(element, "aria-selected");
}
function computeAriaBusy(element) {
  return element.getAttribute("aria-busy") === "true";
}
function computeAriaChecked(element) {
  if ("indeterminate" in element && element.indeterminate) {
    return void 0;
  }
  if ("checked" in element) {
    return element.checked;
  }
  return checkBooleanAttribute(element, "aria-checked");
}
function computeAriaPressed(element) {
  return checkBooleanAttribute(element, "aria-pressed");
}
function computeAriaCurrent(element) {
  var _a, _b;
  return (_b = (_a = checkBooleanAttribute(element, "aria-current")) != null ? _a : element.getAttribute("aria-current")) != null ? _b : false;
}
function computeAriaExpanded(element) {
  if (isDetailsElement(element)) {
    return element.open;
  }
  return checkBooleanAttribute(element, "aria-expanded");
}
function computeRoles(element, isNonLandmarkSubtree) {
  let roles = [];
  if (element.hasAttribute("role")) {
    roles = element.getAttribute("role").split(" ").slice(0, 1);
  } else {
    roles = getImplicitAriaRoles(element, isNonLandmarkSubtree);
  }
  return roles;
}

// src/accessibility-tree.ts
import {
  computeAccessibleDescription,
  computeAccessibleName,
  isInaccessible
} from "dom-accessibility-api";
var isNonLandmarkRole = (element, role) => ["article", "aside", "main", "nav", "section"].includes(
  element.tagName.toLowerCase()
) || ["aricle", "complementary", "main", "navigation", "region"].includes(role);
var defaultOptions = {
  isNonLandmarkSubtree: false
};
var getAccessibilityTree = (element, {
  isNonLandmarkSubtree: userNonLandmarkSubtree = defaultOptions.isNonLandmarkSubtree
} = defaultOptions) => {
  function assembleTree(element2, { nonLandmarkSubtree }) {
    if (isInaccessible(element2)) {
      return null;
    }
    const [role] = computeRoles(element2, nonLandmarkSubtree);
    const childNodes = Array.from(element2.childNodes);
    return {
      element: element2,
      role,
      name: computeAccessibleName(element2),
      description: computeAccessibleDescription(element2),
      state: {
        busy: computeAriaBusy(element2),
        checked: computeAriaChecked(element2),
        current: computeAriaCurrent(element2),
        disabled: element2 == null ? void 0 : element2.disabled,
        expanded: computeAriaExpanded(element2),
        pressed: computeAriaPressed(element2),
        selected: computeAriaSelected(element2)
      },
      children: (childNodes != null ? childNodes : []).map((child) => {
        if (child instanceof HTMLElement) {
          return assembleTree(child, {
            nonLandmarkSubtree: nonLandmarkSubtree || isNonLandmarkRole(element2, role)
          });
        }
        if (child instanceof Text) {
          return new StaticText(child.textContent);
        }
      }).filter(isDefined)
    };
  }
  return assembleTree(element, { nonLandmarkSubtree: userNonLandmarkSubtree });
};

// src/helpers.ts
var containerAttributeValues = {
  name: "",
  role: "generic",
  description: ""
};
var defaultState = {
  busy: false,
  checked: void 0,
  current: false,
  disabled: void 0,
  expanded: void 0,
  pressed: void 0,
  selected: void 0
};
var isDefaultState = (state) => {
  return Object.entries(state).every(
    ([key, value]) => value === defaultState[key]
  );
};
var isDefaultAttributeValues = (node) => {
  return Object.entries(containerAttributeValues).every(
    ([key, value]) => value === node[key]
  );
};
var isContainer = (node) => {
  return isDefaultAttributeValues(node) && isDefaultState(node.state);
};
var getExpectedStringMatch = (receivedValue, expectedValue) => {
  if (expectedValue === void 0) {
    return receivedValue;
  }
  if (expectedValue instanceof RegExp) {
    return expectedValue.test(receivedValue) ? receivedValue : expectedValue;
  }
  return expectedValue;
};

// src/prune-container-nodes.ts
var pruneContainerNodes = (node) => {
  const flatten = (children) => {
    return children.flatMap(
      (child) => {
        if (child instanceof StaticText) {
          return [child];
        } else if (isContainer(child) && child.children) {
          return flatten(child.children);
        } else if (child.children) {
          return [__spreadProps(__spreadValues({}, child), { children: flatten(child.children) })];
        } else {
          return [child];
        }
      }
    );
  };
  return __spreadProps(__spreadValues({}, node), { children: node.children ? flatten(node.children) : [] });
};

// src/prepare-diff.ts
var getReceivedName = (node) => {
  if (node instanceof StaticText || !node) {
    return "";
  }
  return node.name;
};
var getReceivedRole = (node) => {
  var _a;
  if (node instanceof StaticText || !node) {
    return "";
  }
  return (_a = node.role) != null ? _a : void 0;
};
var getReceivedDescription = (node) => {
  if (node instanceof StaticText || !node) {
    return "";
  }
  return node.description;
};
var getReceivedState = (node) => {
  if (node instanceof StaticText || !node) {
    return {};
  }
  return node.state;
};
var getReceivedChildren = (node) => {
  var _a;
  if (node instanceof StaticText || !node) {
    return [];
  }
  return (_a = node.children) != null ? _a : [];
};
var matchToNode = (received, expected) => {
  var _a, _b, _c, _d;
  const name = getExpectedStringMatch(
    getReceivedName(received),
    expected == null ? void 0 : expected.name
  );
  const role = (_a = expected == null ? void 0 : expected.role) != null ? _a : getReceivedRole(received);
  const description = getExpectedStringMatch(
    getReceivedDescription(received),
    expected == null ? void 0 : expected.description
  );
  const state = __spreadValues(__spreadValues({}, getReceivedState(received)), expected == null ? void 0 : expected.state);
  const receivedChildren = getReceivedChildren(received);
  const expectedChildren = expected == null ? void 0 : expected.children;
  if (!expectedChildren) {
    return {
      role,
      name,
      description,
      state,
      children: receivedChildren
    };
  }
  const maxOfChildrenLengths = Math.max(
    (_c = (_b = expected == null ? void 0 : expected.children) == null ? void 0 : _b.length) != null ? _c : 0,
    (_d = receivedChildren.length) != null ? _d : 0
  );
  const children = Array.from({ length: maxOfChildrenLengths }).map((_, index) => {
    var _a2;
    const receivedChild = receivedChildren[index];
    const expectedChild = (_a2 = expected == null ? void 0 : expected.children) == null ? void 0 : _a2[index];
    if (typeof expectedChild === "string") {
      return new StaticText(expectedChild);
    }
    if (expectedChild instanceof RegExp) {
      if (receivedChild instanceof StaticText && receivedChild.text && expectedChild.test(receivedChild.text)) {
        return receivedChild;
      }
      return expectedChild;
    }
    if (expectedChild === void 0) {
      if (name && maxOfChildrenLengths === 1) {
        if (name instanceof RegExp) {
          return name;
        }
        return new StaticText(name);
      }
      return void 0;
    }
    return matchToNode(receivedChild, expectedChild);
  }).filter(Boolean);
  const result = expected ? {
    role,
    name,
    description,
    state,
    children
  } : void 0;
  return result;
};

// src/pretty-tree.ts
var getStateDetails = (state) => {
  const nonDefaultEntries = Object.entries(state).filter(
    ([key, value]) => value !== defaultState[key]
  );
  return nonDefaultEntries.length > 0 ? `{ ${nonDefaultEntries.map(([key, value]) => `${key}: ${value}`).join(", ")} }` : "";
};
var renderStringMatcher = (str) => {
  if (str instanceof RegExp) {
    return str.toString();
  }
  return `"${str}"`;
};
var getPrettyTree = (tree, depth = 0) => {
  const indentation = "  ".repeat(depth);
  if (typeof tree === "string") {
    return `${indentation}"${tree}"
`;
  }
  if (tree instanceof RegExp) {
    return `${indentation}${tree.toString()}
`;
  }
  if (tree instanceof StaticText) {
    return `${indentation}StaticText "${tree.text}"
`;
  }
  if (tree === void 0) {
    return `${indentation}undefined
`;
  }
  const nameString = tree.name ? ` ${renderStringMatcher(tree.name)}` : "";
  const stateString = getStateDetails(tree.state);
  let output = `${indentation}${tree.role}${nameString}${stateString ? " " + stateString : ""}
`;
  if (tree.description) {
    output += `${indentation}  description: ${renderStringMatcher(
      tree.description
    )}
`;
  }
  if (tree.children) {
    tree.children.forEach((child) => {
      output += getPrettyTree(child, depth + 1);
    });
  }
  return output;
};

// src/matchers.ts
function staticTextMatcher(a, b) {
  const isAStaticText = a instanceof StaticText;
  const isBStringMatcher = b instanceof RegExp || typeof b === "string";
  const isSuitableMatcher = isAStaticText && isBStringMatcher;
  if (!isSuitableMatcher) {
    return void 0;
  }
  if (a.text === null) {
    return false;
  }
  if (b instanceof RegExp) {
    return b.test(a.text);
  }
  return a.text === b;
}
function treeNodeMatcher(a, b, customTesters) {
  const isAA11yTreeNode = isA11yTreeNode(a);
  const isBA11yTreeNodeMatch = isA11yTreeNodeMatch(b);
  const isSuitableMatcher = isAA11yTreeNode && isBA11yTreeNodeMatch;
  if (!isSuitableMatcher) {
    return void 0;
  }
  let isEqual = true;
  if (b.name !== void 0) {
    isEqual && (isEqual = b.name instanceof RegExp ? b.name.test(a.name) : a.name === b.name);
  }
  if (b.role !== void 0) {
    isEqual && (isEqual = a.role === b.role);
  }
  if (b.description !== void 0) {
    isEqual && (isEqual = b.description instanceof RegExp ? b.description.test(a.description) : a.description === b.description);
  }
  if (b.state !== void 0) {
    isEqual && (isEqual = this.equals(a.state, b.state, customTesters));
  }
  if (b.children !== void 0) {
    if (a.children === void 0) {
      throw new Error("treeNodeMatcher: a.children is undefined");
    }
    if (a.children.length !== b.children.length) {
      return false;
    }
    for (let i = 0; i < a.children.length; i++) {
      const child = a.children[i];
      const childMatcher = b.children[i];
      isEqual && (isEqual = this.equals(child, childMatcher, customTesters));
    }
  }
  return isEqual;
}
var prepareA11yTreeNodeMatch = (_a) => {
  var _b = _a, {
    children,
    state
  } = _b, matchTreeObject = __objRest(_b, [
    "children",
    "state"
  ]);
  const result = __spreadValues({}, matchTreeObject);
  if (state) {
    result.state = expect.objectContaining(state);
  }
  if (children) {
    result.children = children.map((child) => {
      if (typeof child === "string" || child instanceof RegExp) {
        return child;
      }
      return prepareA11yTreeNodeMatch(child);
    });
  }
  return result;
};
expect.extend({
  toHaveA11yTree(received, expected, options) {
    if (received instanceof HTMLElement) {
      const tree = getAccessibilityTree(received, options);
      if (tree === null) {
        return {
          message: () => "Failed to get accessible tree",
          pass: false
        };
      }
      const flatTree = pruneContainerNodes(tree);
      if (flatTree === null) {
        return {
          message: () => "Failed to flatten accessible tree",
          pass: false
        };
      }
      received = flatTree;
    }
    const expectedResult = prepareA11yTreeNodeMatch(expected);
    const pass = this.equals(received, expectedResult, [
      staticTextMatcher,
      treeNodeMatcher
    ]);
    const expectedPreparedForDiff = matchToNode(received, expected);
    const expectedPrettyTree = getPrettyTree(expectedPreparedForDiff);
    const receivedPrettyTree = getPrettyTree(received);
    if (pass) {
      return {
        message: () => `${this.utils.diff(expectedPrettyTree, receivedPrettyTree)}`,
        pass: true
      };
    }
    return {
      message: () => `${this.utils.diff(expectedPrettyTree, receivedPrettyTree)}`,
      pass: false
    };
  }
});

// src/test-helpers.ts
function byRole(role, nameOrPropertiesOrChildren, childrenIfProvided) {
  const a11yNode = {
    role
  };
  if (typeof nameOrPropertiesOrChildren === "undefined") {
    return a11yNode;
  } else if (
    // byRole with role and name
    typeof nameOrPropertiesOrChildren === "string" || nameOrPropertiesOrChildren instanceof RegExp
  ) {
    a11yNode.name = nameOrPropertiesOrChildren;
    if (Array.isArray(childrenIfProvided)) {
      a11yNode.children = childrenIfProvided;
    }
  } else if (Array.isArray(nameOrPropertiesOrChildren)) {
    a11yNode.children = nameOrPropertiesOrChildren;
  } else if (typeof nameOrPropertiesOrChildren === "object") {
    const _a = nameOrPropertiesOrChildren, { name, description } = _a, state = __objRest(_a, ["name", "description"]);
    a11yNode.name = name;
    a11yNode.description = description;
    a11yNode.state = state;
    if (Array.isArray(childrenIfProvided)) {
      a11yNode.children = childrenIfProvided;
    }
  }
  return a11yNode;
}
export {
  byRole,
  pruneContainerNodes as flattenAccessibilityTree,
  getAccessibilityTree
};
//# sourceMappingURL=index.js.map